import { identity, isNil } from 'ibm-wch-sdk-utils';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * @record
 */

/**
 * @record
 */

/**
 * Converts an RGB color value to HSL. Conversion formula
 * adapted from http://en.wikipedia.org/wiki/HSL_color_space.
 * Assumes r, g, and b are contained in the set [0, 255] and
 * returns h, s, and l in the set [0, 1].
 *
 * @param {?} rgb
 * @return {?} Array           The HSL representation
 */
function rgbaToHsla(rgb) {
    const /** @type {?} */ r = rgb.r / 255, /** @type {?} */ g = rgb.g / 255, /** @type {?} */ b = rgb.b / 255;
    const /** @type {?} */ max = Math.max(r, g, b), /** @type {?} */ min = Math.min(r, g, b);
    const /** @type {?} */ l = (max + min) / 2;
    let /** @type {?} */ h, /** @type {?} */ s;
    if (max == min) {
        h = s = 0; // achromatic
    }
    else {
        var /** @type {?} */ d = max - min;
        s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
        switch (max) {
            case r:
                h = (g - b) / d + (g < b ? 6 : 0);
                break;
            case g:
                h = (b - r) / d + 2;
                break;
            case b:
                h = (r - g) / d + 4;
                break;
        }
        h /= 6;
    }
    return { h, s, l, a: rgb.a };
}
/**
 * @param {?} rgb
 * @return {?}
 */
function rgbaToString(rgb) {
    // convert
    return `rgba(${rgb.r}, ${rgb.g}, ${rgb.b}, ${rgb.a})`;
}
const colorValue = (aKey, aColor) => ({ [`--${aKey}`]: rgbaToString(aColor) });
const BLACK = { r: 0, g: 0, b: 0, a: 1 };
const noParser = color => BLACK;
/**
 * Constructs a function that parses a color value using the DOM APIs
 *
 * @param aDoc the document
 * @return the parser function
 */
const colorParser = aDoc => {
    // sanity check
    if (isNil(aDoc)) {
        return noParser;
    }
    // create the canvas only once
    const /** @type {?} */ cvs = aDoc.createElement('canvas');
    cvs.height = 1;
    cvs.width = 1;
    // the context
    const /** @type {?} */ ctx = cvs.getContext('2d');
    // convert
    return aColor => {
        try {
            // set the fill style
            ctx.fillStyle = aColor;
            ctx.fillRect(0, 0, 1, 1);
            // decode
            const /** @type {?} */ res = ctx.getImageData(0, 0, 1, 1).data;
            // convert
            return {
                r: res[0], g: res[1], b: res[2], a: res[3] / 255
            };
        }
        catch (/** @type {?} */ error) {
            // bail our
            return BLACK;
        }
    };
};
/**
 * Decompose the color string into color components
 *
 * @param aKey the name of the color
 * @param aColor the actual color value
 *
 * @return the styles
 */
const decomposeColor = (aKey, aColor) => {
    // decompose into rgb and hsl
    const /** @type {?} */ hsla = rgbaToHsla(aColor);
    // compose
    return {
        // rgb
        [`--${aKey}-r`]: aColor.r,
        [`--${aKey}-g`]: aColor.g,
        [`--${aKey}-b`]: aColor.b,
        // hsl
        [`--${aKey}-h`]: hsla.h * 360,
        [`--${aKey}-s`]: hsla.s,
        [`--${aKey}-l`]: hsla.l,
        // alpha channel is common
        [`--${aKey}-a`]: aColor.a,
    };
};
const colorVariations = (aKey, aColor) => (Object.assign({}, colorValue(aKey, aColor), decomposeColor(aKey, aColor)));
// expose simple color transforms
const colorToRGBA = identity;
const colorToHSLA = rgbaToHsla;

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
const COLOR_SUFFIX = 'Color';
const _colorName = name => name.substr(0, name.length - COLOR_SUFFIX.length);
/**
 * Derives styles based on a rendering context assuming that the contet represents a theme configuration. Color names
 * are expected to be represented as elements of type 'text'. The element label has to end with the term 'Color', the prefix
 * will be the name of the actual CSS color variable (e.g. 'primaryColor').
 *
 * @param aRenderingContext the rendering context of the theme
 *
 * @return a styles object with the computed styles
 */
const stylesFromRenderingContext = (aRenderingContext, aParser) => {
    // potential colors
    const /** @type {?} */ colors = aRenderingContext.text || {};
    // iterate
    return Object.assign({}, ...Object.keys(colors)
        .filter(name => name.endsWith(COLOR_SUFFIX))
        .map(name => colorVariations(_colorName(name), aParser(colors[name]))));
};

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/* Copyright IBM Corp. 2017 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes} checked by tsc
 */
/**
 * Generated bundle index. Do not edit.
 */

export { colorParser, colorVariations, colorToRGBA, colorToHSLA, stylesFromRenderingContext };
//# sourceMappingURL=ibm-wch-sdk-theme.js.map
