import { __spread } from 'tslib';
import { identity, isNil } from 'ibm-wch-sdk-utils';

function rgbaToHsla(rgb) {
    var r = rgb.r / 255, g = rgb.g / 255, b = rgb.b / 255;
    var max = Math.max(r, g, b), min = Math.min(r, g, b);
    var l = (max + min) / 2;
    var h, s;
    if (max == min) {
        h = s = 0;
    }
    else {
        var d = max - min;
        s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
        switch (max) {
            case r:
                h = (g - b) / d + (g < b ? 6 : 0);
                break;
            case g:
                h = (b - r) / d + 2;
                break;
            case b:
                h = (r - g) / d + 4;
                break;
        }
        h /= 6;
    }
    return { h: h, s: s, l: l, a: rgb.a };
}
function rgbaToString(rgb) {
    return "rgba(" + rgb.r + ", " + rgb.g + ", " + rgb.b + ", " + rgb.a + ")";
}
var colorValue = function (aKey, aColor) {
    return (_a = {}, _a["--" + aKey] = rgbaToString(aColor), _a);
    var _a;
};
var BLACK = { r: 0, g: 0, b: 0, a: 1 };
var noParser = function (color) { return BLACK; };
var colorParser = function (aDoc) {
    if (isNil(aDoc)) {
        return noParser;
    }
    var cvs = aDoc.createElement('canvas');
    cvs.height = 1;
    cvs.width = 1;
    var ctx = cvs.getContext('2d');
    return function (aColor) {
        try {
            ctx.fillStyle = aColor;
            ctx.fillRect(0, 0, 1, 1);
            var res = ctx.getImageData(0, 0, 1, 1).data;
            return {
                r: res[0], g: res[1], b: res[2], a: res[3] / 255
            };
        }
        catch (error) {
            return BLACK;
        }
    };
};
var decomposeColor = function (aKey, aColor) {
    var hsla = rgbaToHsla(aColor);
    return _a = {}, _a["--" + aKey + "-r"] = aColor.r, _a["--" + aKey + "-g"] = aColor.g, _a["--" + aKey + "-b"] = aColor.b, _a["--" + aKey + "-h"] = hsla.h * 360, _a["--" + aKey + "-s"] = hsla.s, _a["--" + aKey + "-l"] = hsla.l, _a["--" + aKey + "-a"] = aColor.a, _a;
    var _a;
};
var colorVariations = function (aKey, aColor) { return (Object.assign({}, colorValue(aKey, aColor), decomposeColor(aKey, aColor))); };
var colorToRGBA = identity;
var colorToHSLA = rgbaToHsla;
var COLOR_SUFFIX = 'Color';
var _colorName = function (name) { return name.substr(0, name.length - COLOR_SUFFIX.length); };
var stylesFromRenderingContext = function (aRenderingContext, aParser) {
    var colors = aRenderingContext.text || {};
    return Object.assign.apply(Object, __spread([{}], Object.keys(colors)
        .filter(function (name) { return name.endsWith(COLOR_SUFFIX); })
        .map(function (name) { return colorVariations(_colorName(name), aParser(colors[name])); })));
};

export { colorParser, colorVariations, colorToRGBA, colorToHSLA, stylesFromRenderingContext };
//# sourceMappingURL=ibm-wch-sdk-theme.js.map
